==> dt-common-0.5.13.tgz: package/redis-tools.js <==
'use strict';

const assert = require('assert');

const redis = require('ioredis');
const RedLock = require('redlock');
const { logError } = require('./logger');

const {
  REDIS_CONNECTION_STRING = 'redis://127.0.0.1:6379',
} = process.env;

const redis_data = redis.createClient(REDIS_CONNECTION_STRING);
const redis_pub = redis.createClient(REDIS_CONNECTION_STRING);
const redis_sub = redis.createClient(REDIS_CONNECTION_STRING);

const redlock = new RedLock(
  // you should have one client for each independent redis node
  // or cluster
  [redis_pub],
  {
    // the expected clock drift; for more details
    // see http://redis.io/topics/distlock
    driftFactor: 0.01, // time in ms

    // the max number of times Redlock will attempt
    // to lock a resource before erroring
    retryCount: 0,

    // the time in ms between attempts
    // retryDelay:  200, // time in ms

    // the max time in ms randomly added to retries
    // to improve performance under high contention
    // see https://www.awsarchitectureblog.com/2015/03/backoff.html
    // retryJitter:  200 // time in ms
  },
);

const mhgetall = async (keys) => {
  try {
    const pipeline = redis_data.pipeline();
    for (const key of keys) {
      pipeline.hgetall(key);
    }

    const data = await pipeline.exec();
    return data.reduce((results, [err, result]) => {
      if (!err) {
        results.push(result);
      }
      return results;
    }, []);
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'mhgetall',
      keys,
    });
  }
};

const redis_listeners = {};

const initRedisSub = (module_id, new_channel_listeners) => {
  try {
    assert.ok(typeof module_id === 'string' && module_id, 'invalid module_id');
    for (const [channel, channelListener] of Object.entries(new_channel_listeners)) {
      assert.ok(typeof channel === 'string' && channel, `invalid channel key: ${channel}`);
      assert.ok(typeof channelListener === 'function', `invalid channelListener: ${channelListener}`);
    }

    if (!redis_listeners[module_id]) {
      redis_listeners[module_id] = {
        messageListener: (channel, message) => {
          const channelListener = redis_listeners[module_id].channel_listeners[channel];
          if (channelListener) {
            channelListener(message);
          }
        },
      };
      redis_sub.on('message', redis_listeners[module_id].messageListener);
    }

    const { channel_listeners={} } = redis_listeners[module_id];
    redis_listeners[module_id].channel_listeners = {
      ...channel_listeners,
      ...Object.entries(new_channel_listeners).reduce((result, [channel, fn]) => {
        result[`${module_id}:${channel}`] = fn;
        return result;
      }, {}),
    };

    for (const channel of Object.keys(new_channel_listeners)) {
      redis_sub.subscribe(`${module_id}:${channel}`);
    }
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'initRedisSub',
      module_id,
      redis_listeners,
    });
  }
};

const killRedisSub = (module_id, channels) => {
  try {
    if (!redis_listeners[module_id]) {
      return;
    }

    const { messageListener, channel_listeners } = redis_listeners[module_id];

    if (channels) {
      // just unsubscribe some channels, but keep the module itself
      assert.ok(Array.isArray(channels) && channels.every(c => typeof c === 'string' && c), `invalid channels arg: ${channels}`);

      for (const channel of channels) {
        redis_sub.unsubscribe(`${module_id}:${channel}`);
        delete channel_listeners[`${module_id}:${channel}`];
      }
    } else {
      // unsubscribe all channel listeners for this module, and delete the module listener
      for (const channel of Object.keys(channel_listeners)) {
        redis_sub.unsubscribe(`${module_id}:${channel}`);
        delete channel_listeners[`${module_id}:${channel}`];
      }

      redis_sub.removeListener('message', messageListener);
      delete redis_listeners[module_id].messageListener;

      delete redis_listeners[module_id];
    }
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'killRedisSub',
      module_id,
    });
  }
};


module.exports = {
  redis_data,
  redis_pub,
  redis_sub,
  redlock,
  mhgetall,
  initRedisSub,
  killRedisSub,
};

==> dt-common-1.1.2.tgz: package/redis-tools.js <==
'use strict';

const assert = require('assert');

const redis = require('ioredis');
const RedLock = require('redlock');
const { logError } = require('./logger');

const { REDIS_CONNECTION_STRING = 'redis://127.0.0.1:6379' } = process.env;

const redis_data = redis.createClient(REDIS_CONNECTION_STRING);
const redis_pub = redis.createClient(REDIS_CONNECTION_STRING);
const redis_sub = redis.createClient(REDIS_CONNECTION_STRING);

const redlock = new RedLock(
  // you should have one client for each independent redis node
  // or cluster
  [redis_pub],
  {
    // the expected clock drift; for more details
    // see http://redis.io/topics/distlock
    driftFactor: 0.01, // time in ms

    // the max number of times Redlock will attempt
    // to lock a resource before erroring
    retryCount: 0,

    // the time in ms between attempts
    // retryDelay:  200, // time in ms

    // the max time in ms randomly added to retries
    // to improve performance under high contention
    // see https://www.awsarchitectureblog.com/2015/03/backoff.html
    // retryJitter:  200 // time in ms
  }
);

const mhgetall = async (keys) => {
  try {
    const pipeline = redis_data.pipeline();
    for (const key of keys) {
      pipeline.hgetall(key);
    }

    const data = await pipeline.exec();
    return data.reduce((results, [err, result]) => {
      if (!err) {
        results.push(result);
      }
      return results;
    }, []);
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'mhgetall',
      keys,
    });
  }
};

const redis_listeners = {};

const initRedisSub = (module_id, new_channel_listeners) => {
  try {
    assert.ok(typeof module_id === 'string' && module_id, 'invalid module_id');
    for (const [channel, channelListener] of Object.entries(
      new_channel_listeners
    )) {
      assert.ok(
        typeof channel === 'string' && channel,
        `invalid channel key: ${channel}`
      );
      assert.ok(
        typeof channelListener === 'function',
        `invalid channelListener: ${channelListener}`
      );
    }

    if (!redis_listeners[module_id]) {
      redis_listeners[module_id] = {
        messageListener: (channel, message) => {
          const channelListener =
            redis_listeners[module_id].channel_listeners[channel];
          if (channelListener) {
            channelListener(message);
          }
        },
      };
      redis_sub.on('message', redis_listeners[module_id].messageListener);
    }

    const { channel_listeners = {} } = redis_listeners[module_id];
    redis_listeners[module_id].channel_listeners = {
      ...channel_listeners,
      ...Object.entries(new_channel_listeners).reduce(
        (result, [channel, fn]) => {
          result[`${module_id}:${channel}`] = fn;
          return result;
        },
        {}
      ),
    };

    for (const channel of Object.keys(new_channel_listeners)) {
      redis_sub.subscribe(`${module_id}:${channel}`);
    }
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'initRedisSub',
      module_id,
      redis_listeners,
    });
  }
};

const killRedisSub = (module_id, channels) => {
  try {
    if (!redis_listeners[module_id]) {
      return;
    }

    const { messageListener, channel_listeners } = redis_listeners[module_id];

    if (channels) {
      // just unsubscribe some channels, but keep the module itself
      assert.ok(
        Array.isArray(channels) &&
          channels.every((c) => typeof c === 'string' && c),
        `invalid channels arg: ${channels}`
      );

      for (const channel of channels) {
        redis_sub.unsubscribe(`${module_id}:${channel}`);
        delete channel_listeners[`${module_id}:${channel}`];
      }
    } else {
      // unsubscribe all channel listeners for this module, and delete the module listener
      for (const channel of Object.keys(channel_listeners)) {
        redis_sub.unsubscribe(`${module_id}:${channel}`);
        delete channel_listeners[`${module_id}:${channel}`];
      }

      redis_sub.removeListener('message', messageListener);
      delete redis_listeners[module_id].messageListener;

      delete redis_listeners[module_id];
    }
  } catch (err) {
    logError(err, {
      module: 'redis-tools',
      func: 'killRedisSub',
      module_id,
    });
  }
};

module.exports = {
  redis_data,
  redis_pub,
  redis_sub,
  redlock,
  mhgetall,
  initRedisSub,
  killRedisSub,
};
